@@ -15,7 +15,7 @@ struct _GdbBreakpoint {
     gchar *type;
     gchar *disposition;
     gboolean enabled;
-    gint address;
+    gchar *address;
     gchar *file_name;
     gint line_num;
 
@@ -29,6 +29,10 @@ const GdbBreakpoint *gdb_breakpoint_set_linenum (GdbInstance *inst,
 const GdbBreakpoint *gdb_breakpoint_set_function (GdbInstance *inst,
 						  const gchar *function_name,
 						  const gchar *condition);
+
+const GdbBreakpoint *gdb_breakpoint_set_watchpoint (GdbInstance *inst, 
+						    const char *expression,
+						    const char *condition);
 const GdbBreakpoint *gdb_breakpoint_get_info (GdbInstance *inst, int bp_num);
 
 gboolean gdb_breakpoint_disable (GdbInstance *inst,
