@@ -42,6 +42,55 @@ gdb_run (GdbInstance *inst)
 		      NULL);
 }
 
+GdbAttachResult
+gdb_attach (GdbInstance *inst, int pid)
+{
+    char *cmd_str;
+    char *line;
+    
+    cmd_str = g_strdup_printf ("attach %d", pid);
+    
+    if (inst->is_running)
+	return GDB_ATTACH_ALREADY_RUNNING;
+
+    gdb_execute_command (inst, cmd_str);
+    g_free (cmd_str);
+
+    /* After executing an attach command, one of two annotations will come
+     * down the pipe -- error-begin on error, and frames-invalid if the load
+     * was successful.  If there was an error, read the error and return.
+     * If frames-invalid is encountered, enter the execution loop (we can do
+     * the execution loop here without using the event loop because an
+     * attach happens relatively quickly). */
+
+    line = gdb_read_line (inst);
+    while (strcmp (line, "\32\32frames-invalid")) {
+	if (!strcmp (line, "\32\32error-begin")) {
+	    line = gdb_read_line (inst);
+	    line += strlen ("ptrace :");
+	    if (!strncmp (line, "No such", strlen ("No such"))) {
+		return GDB_ATTACH_DOESNT_EXIST;
+	    } else if (strncmp (line, "Operation not", strlen ("Operation not"))) {
+		return GDB_ATTACH_ACCESS_DENIED;
+	    }
+	    return GDB_ATTACH_MISC;
+	}
+	line = gdb_read_line (inst);
+    }  
+    inst->is_attached = TRUE;
+    execution_starting (inst);
+    while (!execution_handler (inst, NULL))
+	;
+    return GDB_ATTACH_OK;
+}
+
+void
+gdb_detach (GdbInstance *inst)
+{
+    gdb_execute_command (inst, "detach");
+    inst->is_attached = FALSE;
+}
+
 /* Continues a *running*, but not *executing*, program. */
 void
 gdb_continue (GdbInstance *inst)
