@@ -29,6 +29,11 @@
  * the rest of libdryad.  It isn't too pretty, but the alternative is much
  * worse. */
 
+typedef enum {
+    GDB_SYMBOL_OK,
+    GDB_SYMBOL_INVALID_TYPE
+} GdbSymbolResult;
+
 void gdb_get_locals (GdbInstance *inst, GdfLibdryadSymbolSet *ss);
 void gdb_get_expression (GdbInstance *inst, GDF_Symbol *sym, const char *expr);
 void gdb_expand_symbol (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
@@ -37,6 +42,9 @@ void gdb_update_symbols (GdbInstance *inst, GdfLibdryadSymbolSet *ss);
 GDF_SymbolSequence *gdb_allocate_symbols (void);
 void gdb_add_expression (GdbInstance *inst, GdfLibdryadSymbolSet *set, 
 			 const char *expr);
-
+GdbSymbolResult gdb_set_symbol_value (GdbInstance *inst, GDF_Symbol *sym,
+				      const char *value);
+GdbSymbolResult gdb_cast (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
+			  GDF_Symbol *sym, const char *type);
 
 #endif
