@@ -87,7 +87,7 @@ gdf_libdryad_debugger_new (void)
     debugger->locals = gdf_libdryad_symbol_set_new (debugger);
     debugger->symsets = NULL;
     
-    debugger->frame_changed = FALSE;
+    debugger->frame_changed = -1;
     
     return debugger;
 }
@@ -172,7 +172,7 @@ impl_load_binary (PortableServer_Servant servant,
 	if (dbg->gdb_inst->file_loaded) 
 	    gdb_unload_file (dbg->gdb_inst);
 	
-	dbg->state |= GDF_Debugger_BINARY_LOADED;
+	dbg->state &= ~(GDF_Debugger_BINARY_LOADED);
 	
 	event_any = gdf_marshal_event_none ("program_unloaded");
 	gdf_event_channel_client_push (dbg->event_channel, event_any);
@@ -189,6 +189,9 @@ impl_load_binary (PortableServer_Servant servant,
     success = gdb_load_file (dbg->gdb_inst, file_name);
 
     if (success) {
+
+	dbg->state |= GDF_Debugger_BINARY_LOADED;
+	
 	event_any = gdf_marshal_event_none ("program_loaded");
 	gdf_event_channel_client_push (dbg->event_channel, event_any);
 	CORBA_free (event_any);
@@ -418,6 +421,32 @@ impl_set_breakpoint_function (PortableServer_Servant servant,
     }
 }
 
+static CORBA_long
+impl_set_watchpoint (PortableServer_Servant servant,
+		     const CORBA_char *expr,
+		     const CORBA_char *condition,
+		     CORBA_Environment *ev)
+{
+    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+    CORBA_any *event_any;
+    const GdbBreakpoint *bp;
+
+    g_return_val_if_fail (dbg->gdb_inst != NULL, -1);
+
+    bp = gdb_breakpoint_set_watchpoint (dbg->gdb_inst, expr, 
+					condition[0] ? condition : NULL);
+
+    if (bp) {
+	event_any = gdf_marshal_event_long ("breakpoint_set", (glong)bp->num);
+	gdf_event_channel_client_push (dbg->event_channel, event_any);
+	CORBA_free (event_any);
+	return bp->num;
+    } else {
+	/* FIXME: Set exception */
+	return -1;
+    }
+}
+
 static void
 impl_enable_breakpoint (PortableServer_Servant servant, 
 			CORBA_long bp_num,
@@ -491,10 +520,10 @@ impl_get_breakpoint_info (PortableServer_Servant servant,
     }
     
     ret->num = bp_num;
-    ret->type = CORBA_string_dup (bp->type);
+    ret->type = CORBA_string_dup (bp->type ? bp->type : "");
     ret->enabled = (CORBA_boolean)bp->enabled;
-    ret->address = (CORBA_long)bp->address;
-    ret->file_name = CORBA_string_dup (bp->file_name);
+    ret->address = CORBA_string_dup (bp->address ? bp->address : "");
+    ret->file_name = CORBA_string_dup (bp->file_name ? bp->file_name : "");
     ret->line_num = (CORBA_long)bp->line_num;
 
     return ret;
@@ -785,6 +814,7 @@ init_libdryad_debugger_corba_class (void)
     libdryad_debugger_epv.get_absolute_source_path = impl_get_absolute_source_path;
     libdryad_debugger_epv.set_breakpoint = impl_set_breakpoint;
     libdryad_debugger_epv.set_breakpoint_function = impl_set_breakpoint_function;
+    libdryad_debugger_epv.set_watchpoint = impl_set_watchpoint;
     libdryad_debugger_epv.enable_breakpoint = impl_enable_breakpoint;
     libdryad_debugger_epv.disable_breakpoint = impl_disable_breakpoint;
     libdryad_debugger_epv.delete_breakpoint = impl_delete_breakpoint;
@@ -916,9 +946,12 @@ execution_stopped_cb (GdfLibdryadDebugger *dbg)
 	 * We need to process this here rather than in 
 	 * execution_frame_change_cb because any libdryad interaction in 
 	 * execution_frame_change_cb will screw up the execution parsing. */
-	if (dbg->frame_changed) {
+	if (dbg->frame_changed != -1) {
 	    process_frame_change (dbg);
-	    dbg->frame_changed = FALSE;
+	    event_any = gdf_marshal_event_long ("frame_change",
+						dbg->frame_changed);
+	    dbg->event_queue = g_list_prepend (dbg->event_queue, event_any);
+	    dbg->frame_changed = -1;
 	} else {
 	    gdf_libdryad_debugger_update_symsets (dbg, TRUE);
 	}
@@ -956,12 +989,7 @@ execution_source_line_cb (gchar *source_file, gint line_num,
 static void
 execution_frame_change_cb (int frame_id, GdfLibdryadDebugger *dbg)
 {
-    CORBA_any *event_any;
-
-    event_any = gdf_marshal_event_long ("frame_change", frame_id);
-    dbg->event_queue = g_list_prepend (dbg->event_queue, event_any);
-    
-    dbg->frame_changed = TRUE;
+    dbg->frame_changed = frame_id;
 }
 
 static void
