@@ -58,8 +58,8 @@ static void disconnect_debugger_signals (GdfBreakpointManager *bpm);
 static gint get_row_by_bpnum (GdfBreakpointManager *bpm, gint bp_num);
 static gboolean bp_is_enabled (GdfBreakpointManager *bpm, gint bp_num);
 static void add_breakpoint (GdfBreakpointManager *bpm, gint bp_num,
-							gboolean enabled, GDF_MemoryAddress address, 
-							gchar *filename, gint line_num);
+							gboolean enabled, const char *address, 
+							const char *filename, gint line_num);
 static void enable_breakpoint (GdfBreakpointManager *bpm, gint bp_num);
 static void disable_breakpoint (GdfBreakpointManager *bpm, gint bp_num);
 static void delete_breakpoint (GdfBreakpointManager *bpm, gint bp_num);
@@ -85,7 +85,7 @@ static void breakpoint_deleted_cb (GdfDebuggerClient *dbg, gint bp_num,
 
 
 #define NUM_COLUMNS 5
-static const gint bpm_column_widths[NUM_COLUMNS] = { 10, 30, 55, 35 };
+static const gint bpm_column_widths[NUM_COLUMNS] = { 10, 30, 65, 35 };
 
 static GnomeUIInfo popup_menu[] =
 {
@@ -414,21 +414,20 @@ bp_is_enabled (GdfBreakpointManager *bpm, gint bp_num)
 
 void 
 add_breakpoint (GdfBreakpointManager *bpm, gint bp_num,
-				gboolean enabled, GDF_MemoryAddress address, gchar *filename,
+				gboolean enabled, const char *address, const char *filename,
 				gint line_num)
 {
 	gchar *text[6];
 	
 	text[0] = g_strdup_printf ("%d", bp_num);
 	text[1] = enabled ? _("Y") : _("N");
-	text[2] = g_strdup_printf ("%p", (gpointer)address);
+	text[2] = (char *)address;
 	text[3] = g_strdup_printf ("%d", line_num);
-	text[4] = filename;
+	text[4] = (char *)filename;
 	
 	gtk_clist_append (GTK_CLIST (bpm->priv->clist), text);
 
 	g_free (text[0]);
-	g_free (text[2]);
 	g_free (text[3]);
 }
 
@@ -597,7 +596,7 @@ breakpoint_set_cb (GdfDebuggerClient *dbg, gint bp_num,
 		filename = bp->file_name;
 	
 	add_breakpoint (bpm, bp_num, (gboolean) bp->enabled,
-					bp->address, (gchar*)filename, bp->line_num);
+					bp->address, filename, bp->line_num);
 
 	CORBA_free (bp);
 }
