@@ -34,6 +34,7 @@ struct _GdfSourceViewerManagerPriv {
 	GdfSourceViewer *prev_sv;
 	gboolean has_empty_page : 1;
     gint num_pages;
+    int nonexistent_page_num;
 
 	GHashTable *breakpoint_locations;
 
@@ -78,6 +79,8 @@ static GtkWidget *create_source_page (GdfSourceViewerManager *svm,
 									  const gchar *source_path);
 static void create_empty_page (GdfSourceViewerManager *svm);
 static void remove_empty_page (GdfSourceViewerManager *svm);
+static void create_nonexistent_page (GdfSourceViewerManager *svm,
+                                     char *frame_name);
 static gint find_opened_page (GdfSourceViewerManager *svm, 
 							  const gchar *source_path);
 static void remove_page (GdfSourceViewerManager *svm, gint page_num);
@@ -236,6 +239,8 @@ source_viewer_manager_init (GdfSourceViewerManager *svm)
     GtkWidget *label;
 	svm->priv = g_new0 (GdfSourceViewerManagerPriv, 1);
 
+    svm->priv->nonexistent_page_num = -1;
+
 	svm->priv->notebook = gtk_notebook_new ();
     gtk_notebook_set_scrollable (GTK_NOTEBOOK (svm->priv->notebook), TRUE);
     gtk_notebook_set_show_tabs (GTK_NOTEBOOK (svm->priv->notebook), FALSE);
@@ -315,7 +320,6 @@ set_arg (GtkObject *object,
     
     switch (arg_id) {
     case ARG_DEBUGGER :
-        GDF_TRACE ();
         gdf_source_viewer_manager_set_debugger (svm,
                                                 GTK_VALUE_OBJECT (*arg) ? GDF_DEBUGGER_CLIENT (GTK_VALUE_OBJECT (*arg)) : NULL);
         break;
@@ -438,7 +442,8 @@ view_source (GdfSourceViewerManager *svm, const gchar *source_path)
 
         } else {
             gtk_notebook_set_page (GTK_NOTEBOOK (svm->priv->notebook), 
-                                   svm->priv->num_pages - 1);
+                                   gtk_notebook_page_num (GTK_NOTEBOOK (svm->priv->notebook), 
+                                                          ret));
         }
 	}
 
@@ -510,6 +515,37 @@ remove_empty_page (GdfSourceViewerManager *svm)
     }
 }
 
+void
+create_nonexistent_page (GdfSourceViewerManager *svm,
+                         char *frame_name)
+{
+    char *label_text;
+    label_text = g_strdup_printf (_("No source file available for frame: %s"),
+                                  frame_name);
+    if (svm->priv->nonexistent_page_num == -1) {
+        GtkWidget *tab_label = gtk_label_new (_("No File"));
+        GtkWidget *child_label = gtk_label_new (label_text);
+        
+        gtk_notebook_append_page (GTK_NOTEBOOK (svm->priv->notebook), 
+                                  child_label, tab_label);
+        
+        gtk_widget_show (tab_label);
+        gtk_widget_show (child_label);
+        svm->priv->nonexistent_page_num = 
+            gtk_notebook_page_num (GTK_NOTEBOOK (svm->priv->notebook), 
+                                   child_label);
+    } else {
+        GtkWidget *label = 
+            gtk_notebook_get_nth_page (GTK_NOTEBOOK (svm->priv->notebook),
+                                       svm->priv->nonexistent_page_num);
+        gtk_label_set_text (GTK_LABEL (label), label_text);
+    }
+
+    gtk_notebook_set_page (GTK_NOTEBOOK (svm->priv->notebook), 
+                           svm->priv->nonexistent_page_num);
+
+    g_free (label_text);
+}
 
 gint 
 find_opened_page (GdfSourceViewerManager *svm, 
@@ -532,7 +568,7 @@ find_opened_page (GdfSourceViewerManager *svm,
                                         page);
 		full_filename =
 			gtk_object_get_data (GTK_OBJECT (label), "full_filename");
-		if (strcmp (full_filename, source_path) == 0) {
+		if (full_filename && strcmp (full_filename, source_path) == 0) {
 			return cur;
 		}
 		
@@ -566,6 +602,9 @@ add_breakpoint (GdfSourceViewerManager *svm,
 	GtkWidget *sv;
     char *filename = NULL;
     
+    if (!bp->file_name[0])
+        return;
+
     gdf_debugger_client_get_absolute_source_path (svm->priv->dbg, 
                                                   bp->file_name,
                                                   &filename);
@@ -594,6 +633,9 @@ enable_breakpoint (GdfSourceViewerManager *svm, gint bp_num, gboolean enabled)
 	source_path = g_hash_table_lookup (svm->priv->breakpoint_locations, 
 									   GINT_TO_POINTER (bp_num));
 
+    if (!source_path) 
+        return;
+
 	sv = view_source (svm, source_path);
     
     if (!sv) 
@@ -626,6 +668,9 @@ remove_breakpoint (GdfSourceViewerManager *svm, gint bp_num)
 	source_path = g_hash_table_lookup (svm->priv->breakpoint_locations,
 									   GINT_TO_POINTER (bp_num));
 
+    if (!source_path)
+        return;
+
 	/* Only send this event to opened sources - don't bother reopening a
      * breakpoint just to delete it */
 	if ((page = find_opened_page (svm, source_path)) > -1) {
@@ -801,7 +846,6 @@ void
 execution_exited_cb (GdfDebuggerClient *dbg, gint exit_code,
                      GdfSourceViewerManager *svm)
 {
-    GDF_TRACE ();
 	if (svm->priv->prev_sv)
 		gdf_source_viewer_clear_current_line (svm->priv->prev_sv);
 	
@@ -815,18 +859,25 @@ frame_changed_cb (GdfDebuggerClient *dbg, gint new_frame,
     GDF_StackFrame *frame = NULL;
     GtkWidget *sv;
     char *full_path = NULL;
-
+    
     gdf_debugger_client_get_frame (dbg, new_frame, &frame);
-    gdf_debugger_client_get_absolute_source_path (dbg, 
-                                                  frame->location.file,
-                                                  &full_path);
-
-    sv = view_source (svm, full_path);
-    gdf_source_viewer_view_line (GDF_SOURCE_VIEWER (sv), 
-                                 frame->location.line);
+    
+    if (frame->location.file[0]) {
+        gdf_debugger_client_get_absolute_source_path (dbg, 
+                                                      frame->location.file,
+                                                      &full_path);
 
+    
+        
+        sv = view_source (svm, full_path);
+        gdf_source_viewer_view_line (GDF_SOURCE_VIEWER (sv), 
+                                     frame->location.line);
+        g_free (full_path);
+    } else {
+        create_nonexistent_page (svm, frame->function);
+    }
+    
     CORBA_free (frame);
-    g_free (full_path);
 }
 
 /* Breakpoint hash functions */
