@@ -87,6 +87,8 @@ static void set_value_selected_cb (GtkWidget *widget,
                                    GdfVariableViewer *vv);
 static void cast_selected_cb (GtkWidget *widget, 
                               GdfVariableViewer *vv);
+static void watchpoint_selected_cb (GtkWidget *widget, 
+                                    GdfVariableViewer *vv);
 
 static GnomeUIInfo popup_menu[] =
 {
@@ -112,6 +114,17 @@ static GnomeUIInfo popup_menu[] =
         GNOME_STOCK_MENU_CONVERT,
         0, 0, NULL
     }, 
+    {
+        GNOME_APP_UI_ITEM,
+        N_("Set watchpoint"),
+        N_("Set a watchpoint on the expression"),
+        watchpoint_selected_cb,
+        NULL,
+        NULL,
+        GNOME_APP_PIXMAP_STOCK,
+        GNOME_STOCK_MENU_SEARCH,
+        0, 0, NULL
+    },
     GNOMEUIINFO_END
 };
 
@@ -790,7 +803,7 @@ set_value_selected_cb (GtkWidget *widget,
 
 void 
 cast_selected_cb (GtkWidget *widget, 
-                       GdfVariableViewer *vv)
+                  GdfVariableViewer *vv)
 {
     int sym_id;
     GDF_Symbol *sym;
@@ -861,3 +874,23 @@ cast_selected_cb (GtkWidget *widget,
     }
 }
 
+void
+watchpoint_selected_cb (GtkWidget *widget, 
+                        GdfVariableViewer *vv)
+{
+    int sym_id;
+    GDF_Symbol *sym;
+    GdfDebuggerClientResult res;
+    
+    sym_id = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (vv), 
+                                                   "symbol_id"));
+    gdf_symbol_set_client_get_symbol (vv->priv->ss, sym_id, &sym);
+    
+    res = gdf_debugger_client_set_watchpoint (vv->priv->dbg, 
+                                              sym->expression, 
+                                              NULL, NULL);
+
+    CORBA_free (sym);
+}
+
+
