@@ -23,6 +23,7 @@
 
 #include <config.h>
 #include "gdf-debugger-client.h"
+#include "gdf-tools.h"
 
 #include <liboaf/liboaf.h>
 #include "gdf-event-marshallers.h"
@@ -99,6 +100,33 @@ gdf_debugger_client_construct (GdfDebuggerClient *dbg,
                         (gpointer)dbg);  
 }
 
+GdfDebuggerClient *
+gdf_debugger_client_new_for_mime_type (const char *type)
+{
+    GdfDebuggerClient *ret = NULL;
+    CORBA_Environment ev;
+    char *query;
+    OAF_ServerInfoList *oaf_result;
+    
+    CORBA_exception_init (&ev);
+    
+    query = g_strdup_printf ("repo_ids.has ('IDL:GDF/Debugger:1.0') AND supported_mime_types.has ('%s')", type ? type : "unknown");
+    
+    oaf_result = oaf_query (query, NULL, &ev);
+
+    /* FIXME: Deal with the case where a mime type is handled by more than
+     * one backend */
+    if (ev._major == CORBA_NO_EXCEPTION && oaf_result != NULL && oaf_result->_length >= 1) {
+        ret = gdf_debugger_client_new (oaf_result->_buffer[0].iid);
+    }
+
+    if (oaf_result != NULL) {
+        CORBA_free (oaf_result);
+    }
+
+    g_free (query);
+    return ret;
+}
 
 /**
  * gdf_debugger_client_new:
@@ -493,6 +521,41 @@ gdf_debugger_client_set_breakpoint_function (GdfDebuggerClient *dbg,
     return res;
 }
 
+GdfDebuggerClientResult
+gdf_debugger_client_set_watchpoint (GdfDebuggerClient *dbg,
+                                    const char *expr,
+                                    const char *condition,
+                                    long *bp_num)
+{
+    CORBA_Environment ev;
+    GdfDebuggerClientResult res;
+    long num;
+    
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (expr != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    
+    CORBA_exception_init (&ev);
+    
+    num = GDF_Debugger_set_watchpoint (dbg->objref, expr, 
+                                       condition ? condition : "",
+                                       &ev);
+    
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        num = -1;
+    }
+
+    if (bp_num) {
+        *bp_num = num;
+    }
+    
+    CORBA_exception_free (&ev);
+    return res;
+}
+
+
 GdfDebuggerClientResult
 gdf_debugger_client_enable_breakpoint (GdfDebuggerClient *dbg,
                                        long bp_num)
