@@ -96,6 +96,7 @@ enum _GdfDebuggerClientResult {
 };
 
 GdfDebuggerClient *gdf_debugger_client_new (const char *iid);
+GdfDebuggerClient *gdf_debugger_client_new_for_mime_type (const char *type);
 GdfDebuggerClient *gdf_debugger_client_new_from_corba (GDF_Debugger debugger);
 GtkType gdf_debugger_client_get_type (void);
 
@@ -128,7 +129,10 @@ GdfDebuggerClientResult  gdf_debugger_client_set_breakpoint_function (GdfDebugge
                                                                       const char *function_name,
                                                                       const char *condition,
                                                                       long *bp_num);
-
+GdfDebuggerClientResult gdf_debugger_client_set_watchpoint (GdfDebuggerClient *dbg,
+                                                            const char *expr,
+                                                            const char *condition,
+                                                            long *bp_num);
 GdfDebuggerClientResult gdf_debugger_client_enable_breakpoint (GdfDebuggerClient *dbg,
                                                                long bp_num);
 GdfDebuggerClientResult gdf_debugger_client_disable_breakpoint (GdfDebuggerClient *dbg,
